﻿--create the addon frame----------------------------------
----------------------------------------------------------

	trueitemlevel = CreateFrame("Frame", "trueitemlevel", UIParent);
	trueitemlevel:SetScript('OnEvent', function(self, event, ...) self[event](self, event, ...) end);

	--register events

	trueitemlevel:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
	trueitemlevel:RegisterEvent("PLAYER_LOGIN");

--hook into game scripts----------------------------------
----------------------------------------------------------

	GameTooltip:HookScript("OnShow", function() trueitemlevel:TooltipIsShown() end);
	GameTooltip:HookScript("OnUpdate", function() trueitemlevel:UpdateToolTip() end);

--set constants-------------------------------------------
----------------------------------------------------------

	local locale = GetLocale();
	local playerName, myItemLevel;
	local pauseAddon = false;

--event function library----------------------------------
----------------------------------------------------------

	function trueitemlevel:PLAYER_EQUIPMENT_CHANGED()
		myItemLevel = trueitemlevel:getItemAverage("player");
	end

	function trueitemlevel:PLAYER_LOGIN()
		playerName = UnitName("player");
		myItemLevel = trueitemlevel:getItemAverage("player");

		PAPERDOLL_STATINFO["TRUEITEMLEVEL"] = {
			updateFunc = function(statFrame, unit)
				myItemLevel = trueitemlevel:getItemAverage("player");
				PaperDollFrame_SetLabelAndText(statFrame, "True Item Level", "|cff00ccff"..myItemLevel.."|r")
				statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..format(PAPERDOLLFRAME_TOOLTIP_FORMAT, "真實物品等級 (TiL)")..FONT_COLOR_CODE_CLOSE.." |cff00ccff"..myItemLevel.."|r";
				statFrame.tooltip2 = "Unlike Blizzard's averaged item level, true item level only counts the items you currently have equipped. Heirloom items are not averaged, as they give a deceptively small and inaccurate item level average.";
				statFrame:Show()
			end
		}
		tinsert(PAPERDOLL_STATCATEGORIES["GENERAL"].stats, "TRUEITEMLEVEL")
	end

--misc function library-----------------------------------
----------------------------------------------------------

function trueitemlevel:controlTIL(actioncode) 
	if (actioncode == 0) then
		--pause functions
		DEFAULT_CHAT_FRAME:AddMessage("Pausing True Item Level");
		pauseAddon = true;
	else
		--start functions
		DEFAULT_CHAT_FRAME:AddMessage("Resuming True Item Level");
		pauseAddon = false;
	end
end

function trueitemlevel:TooltipIsShown()
	tooltipshown = true;
end

function trueitemlevel:UpdateToolTip()
	if (InspectFrame and InspectFrame:IsVisible()) then
			tooltipshown = false;
			return
	else
		--we are not inspecting anyone, safe to run code
		
		--[[ Combat Lockdown Code.
		
			if InCombatLockdown() then
				tooltipshown = false;
				return
			end
			
		--]]

		if (InspectFrame and InspectFrame:IsVisible()) then
			tooltipshown = false;
			DEFAULT_CHAT_FRAME:AddMessage("Inspect frame shown, not calculating");
			return
		end

		if (tooltipshown) then
			if not UnitIsPlayer("mouseover") and not UnitInParty("mouseover") and not UnitInRaid("mouseover") then
				--tooltip is not for a player in the world, nor for a unit frame
				return
			end

			local iLVL = 0;
			local BOA = 0;
			local PVP = 0;

			if CanInspect("mouseover") then
				NotifyInspect("mouseover");
				iLVL, BOA, PVP = trueitemlevel:getItemAverage("mouseover");
			end

			if (iLVL > 0 ) then
	
				GameTooltip:AddLine(" ");
				GameTooltip:AddLine("|cffffcc00TiL:|r |cffffffff"..iLVL.."|r");

				if (BOA > 0) then
					GameTooltip:AddLine("|cff777777 •"..BOA.." BOA|r");
				end
	
				if (PVP > 0) then
					GameTooltip:AddLine("|cff777777 •"..PVP.." PVP|r");
				end
				
				tooltipshown = false;
				GameTooltip:Show();
			end
	
			tooltipshown = false;
		end
	end
end

function trueitemlevel:getItemAverage(unit)
	local iLVL, count = 0, 0;
	local BOA = 0;
	local PVP = 0;

	for i = 1,18 do
		if (i == 4) then
			--do nothing
				--slot 4 is for shirts and should not be averaged
				--slot 19 is for tabards, and should not be iterated through
		else
			link = GetInventoryItemLink(unit, i);
			if (link) then
				local iname,_,rarity,level = GetItemInfo(link);
				local stats = GetItemStats(link);
				if (level) then
					--check if boa (heirloom)
					if (rarity == 7) then
						--boa gear
						--boa gear is exempt from averaging
						BOA = BOA + 1;
					else
						--not boa gear, so we can include it in the average
						count = count + 1
						iLVL = iLVL + level

						--check if pvp (has resillience)
						for stat, value in pairs(stats) do
							if (_G[stat] == "Resilience Rating") then
								PVP = PVP + 1;
								break;
							end
						end
					end
				end
			end
		end
	end

	local totalAVG;

	if (count > 0) then
		totalAVG = floor((iLVL / count) + .5);
	else
		totalAVG = 0;
	end

	return totalAVG, BOA, PVP
end